#!/bin/sh
#
# bin/dbanalyscsv.sh
#
# Produce a dbanalys report and CSV files for a database. Requires OE 11.4 or later.
# for older releases, use bin/dbanalys.sh instead.
#
# The CSV format is required byProTop for dbanalys parsing in databases enabled
# for multi-tenancy or table partitioning.
# 
# Usage:
#
#	bin/dbanalyscsv.sh /db/dbname friendlyName
#
# crontab:
#
#	0 1 * * 6 dbanalyscsv.sh /db/sports sports > /tmp/dbanalyscsv.log 2>&1 &
#


. #ENVPT3#/bin/protopenv

TMPDIR=${TMPDIR-/tmp}
LOGDIR=${LOGDIR-/tmp}

errmsg()
{
	echo ""
	echo "${1}"
	echo ""
}

if [ ! -d ${TMPDIR} ]
then
        mkdir ${TMPDIR}
fi

if [ ! -d ${LOGDIR} ]
then
        mkdir ${LOGDIR}
fi

DBPATH=${1}
if [ -z "${DBPATH}" ]
then
	errmsg "Please specify a database path."
	exit
fi

# FRNAME is a "friendly name" in etc/dblist.cfg
#

FRNAME=${2}

if [ -z "${FRNAME}" ]
then
	errmsg "Please specify a friendly name in etc/dblist.cfg."
	exit
fi

# ensure the target directory exists
#

if [ ! -d "${PROTOP}/dbanalys" ]
then
    mkdir ${PROTOP}/dbanalys

	if [ $? -ne 0 ]
	then
	    errmsg "Target directory \"${PROTOP}/dbanalys\" does not exist and could not be created."
    fi
fi

# get a fresh dbanalys
#

if [ ! -f ${DBPATH}.db ]
then
	errmsg "${DBPATH}.db does not exist."
else
    
    cd ${PROTOP}/dbanalys
	${DLC}/bin/proutil ${DBPATH} -C dbanalys -csoutput -verbose -csfilePrefix ${FRNAME} -Bp 10 > ${PROTOP}/dbanalys/${FRNAME}.dba 2> ${TMPDIR}/dbanalys.${FRNAME}.err &
fi
